@@ -140,9 +140,9 @@ func TestLocalRun_stalePlan(t *testing.T) {
 	}
 
 	// Refresh the state
-	sm, err := b.StateMgr("")
-	if err != nil {
-		t.Fatalf("unexpected error: %s", err)
+	sm, sDiags := b.StateMgr("")
+	if sDiags.HasErrors() {
+		t.Fatalf("unexpected error: %s", sDiags.Err())
 	}
 	if err := sm.RefreshState(); err != nil {
 		t.Fatalf("unexpected error refreshing state: %s", err)
@@ -214,7 +214,7 @@ type backendWithStateStorageThatFailsRefresh struct {
 
 var _ backend.Backend = backendWithStateStorageThatFailsRefresh{}
 
-func (b backendWithStateStorageThatFailsRefresh) StateMgr(workspace string) (statemgr.Full, error) {
+func (b backendWithStateStorageThatFailsRefresh) StateMgr(workspace string) (statemgr.Full, tfdiags.Diagnostics) {
 	return &stateStorageThatFailsRefresh{}, nil
 }
 
@@ -230,11 +230,11 @@ func (b backendWithStateStorageThatFailsRefresh) Configure(cty.Value) tfdiags.Di
 	return nil
 }
 
-func (b backendWithStateStorageThatFailsRefresh) DeleteWorkspace(name string, force bool) error {
-	return fmt.Errorf("unimplemented")
+func (b backendWithStateStorageThatFailsRefresh) DeleteWorkspace(name string, force bool) tfdiags.Diagnostics {
+	return tfdiags.Diagnostics{}.Append(fmt.Errorf("unimplemented"))
 }
 
-func (b backendWithStateStorageThatFailsRefresh) Workspaces() ([]string, error) {
+func (b backendWithStateStorageThatFailsRefresh) Workspaces() ([]string, tfdiags.Diagnostics) {
 	return []string{"default"}, nil
 }
 
