@@ -24,6 +24,7 @@ import (
 	"github.com/hashicorp/terraform/internal/states/remote"
 	"github.com/hashicorp/terraform/internal/states/statefile"
 	"github.com/hashicorp/terraform/internal/states/statemgr"
+	"github.com/hashicorp/terraform/internal/tfdiags"
 )
 
 func TestRemoteClient_impl(t *testing.T) {
@@ -48,9 +49,9 @@ func TestRemoteClientBasic(t *testing.T) {
 	createS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 	defer deleteS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 
-	state, err := b.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	state, sDiags := b.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 
 	remote.TestClient(t, state.(*remote.State).Client)
@@ -83,14 +84,14 @@ func TestRemoteClientLocks(t *testing.T) {
 	createDynamoDBTable(ctx, t, b1.dynClient, bucketName)
 	defer deleteDynamoDBTable(ctx, t, b1.dynClient, bucketName)
 
-	s1, err := b1.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, sDiags := b1.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags.Err())
 	}
 
-	s2, err := b2.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s2, sDiags := b2.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags.Err())
 	}
 
 	remote.TestRemoteLocks(t, s1.(*remote.State).Client, s2.(*remote.State).Client)
@@ -125,9 +126,9 @@ func TestForceUnlock(t *testing.T) {
 	defer deleteDynamoDBTable(ctx, t, b1.dynClient, bucketName)
 
 	// first test with default
-	s1, err := b1.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, sDiags := b1.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 
 	info := statemgr.NewLockInfo()
@@ -140,9 +141,9 @@ func TestForceUnlock(t *testing.T) {
 	}
 
 	// s1 is now locked, get the same state through s2 and unlock it
-	s2, err := b2.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal("failed to get default state to force unlock:", err)
+	s2, sDiags := b2.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal("failed to get default state to force unlock:", sDiags)
 	}
 
 	if err := s2.Unlock(lockID); err != nil {
@@ -151,9 +152,9 @@ func TestForceUnlock(t *testing.T) {
 
 	// now try the same thing with a named state
 	// first test with default
-	s1, err = b1.StateMgr("test")
-	if err != nil {
-		t.Fatal(err)
+	s1, sDiags = b1.StateMgr("test")
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 
 	info = statemgr.NewLockInfo()
@@ -166,9 +167,9 @@ func TestForceUnlock(t *testing.T) {
 	}
 
 	// s1 is now locked, get the same state through s2 and unlock it
-	s2, err = b2.StateMgr("test")
-	if err != nil {
-		t.Fatal("failed to get named state to force unlock:", err)
+	s2, sDiags = b2.StateMgr("test")
+	if sDiags.HasErrors() {
+		t.Fatal("failed to get named state to force unlock:", sDiags)
 	}
 
 	if err = s2.Unlock(lockID); err != nil {
@@ -202,9 +203,9 @@ func TestForceUnlock_withLockfile(t *testing.T) {
 	defer deleteS3Bucket(ctx, t, b1.s3Client, bucketName, b1.awsConfig.Region)
 
 	// first test with default
-	s1, err := b1.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, sDiags := b1.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 
 	info := statemgr.NewLockInfo()
@@ -217,9 +218,9 @@ func TestForceUnlock_withLockfile(t *testing.T) {
 	}
 
 	// s1 is now locked, get the same state through s2 and unlock it
-	s2, err := b2.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal("failed to get default state to force unlock:", err)
+	s2, sDiags := b2.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal("failed to get default state to force unlock:", sDiags)
 	}
 
 	if err := s2.Unlock(lockID); err != nil {
@@ -228,9 +229,9 @@ func TestForceUnlock_withLockfile(t *testing.T) {
 
 	// now try the same thing with a named state
 	// first test with default
-	s1, err = b1.StateMgr("test")
-	if err != nil {
-		t.Fatal(err)
+	s1, sDiags = b1.StateMgr("test")
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 
 	info = statemgr.NewLockInfo()
@@ -243,9 +244,9 @@ func TestForceUnlock_withLockfile(t *testing.T) {
 	}
 
 	// s1 is now locked, get the same state through s2 and unlock it
-	s2, err = b2.StateMgr("test")
-	if err != nil {
-		t.Fatal("failed to get named state to force unlock:", err)
+	s2, sDiags = b2.StateMgr("test")
+	if sDiags.HasErrors() {
+		t.Fatal("failed to get named state to force unlock:", sDiags)
 	}
 
 	if err = s2.Unlock(lockID); err != nil {
@@ -272,9 +273,9 @@ func TestRemoteClient_clientMD5(t *testing.T) {
 	createDynamoDBTable(ctx, t, b.dynClient, bucketName)
 	defer deleteDynamoDBTable(ctx, t, b.dynClient, bucketName)
 
-	s, err := b.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s, sDiags := b.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 	client := s.(*remote.State).Client.(*RemoteClient)
 
@@ -322,9 +323,9 @@ func TestRemoteClient_stateChecksum(t *testing.T) {
 	createDynamoDBTable(ctx, t, b1.dynClient, bucketName)
 	defer deleteDynamoDBTable(ctx, t, b1.dynClient, bucketName)
 
-	s1, err := b1.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, sDiags := b1.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 	client1 := s1.(*remote.State).Client
 
@@ -347,30 +348,30 @@ func TestRemoteClient_stateChecksum(t *testing.T) {
 		"bucket": bucketName,
 		"key":    keyName,
 	})).(*Backend)
-	s2, err := b2.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s2, sDiags := b2.StateMgr(backend.DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatal(sDiags)
 	}
 	client2 := s2.(*remote.State).Client
 
 	// write the new state through client2 so that there is no checksum yet
-	if err := client2.Put(newState.Bytes()); err != nil {
-		t.Fatal(err)
+	if diags := client2.Put(newState.Bytes()); diags.HasErrors() {
+		t.Fatal(diags.Err())
 	}
 
 	// verify that we can pull a state without a checksum
-	if _, err := client1.Get(); err != nil {
-		t.Fatal(err)
+	if _, diags := client1.Get(); diags.HasErrors() {
+		t.Fatal(diags.Err())
 	}
 
 	// write the new state back with its checksum
-	if err := client1.Put(newState.Bytes()); err != nil {
-		t.Fatal(err)
+	if diags := client1.Put(newState.Bytes()); diags.HasErrors() {
+		t.Fatal(diags.Err())
 	}
 
 	// put an empty state in place to check for panics during get
-	if err := client2.Put([]byte{}); err != nil {
-		t.Fatal(err)
+	if diags := client2.Put([]byte{}); diags.HasErrors() {
+		t.Fatal(diags.Err())
 	}
 
 	// remove the timeouts so we can fail immediately
@@ -385,27 +386,43 @@ func TestRemoteClient_stateChecksum(t *testing.T) {
 
 	// fetching an empty state through client1 should now error out due to a
 	// mismatched checksum.
-	if _, err := client1.Get(); !IsA[badChecksumError](err) {
-		t.Fatalf("expected state checksum error: got %s", err)
-	} else if bse, ok := As[badChecksumError](err); ok && len(bse.digest) != 0 {
-		t.Fatalf("expected empty checksum, got %x", bse.digest)
+	if _, diags := client1.Get(); diags.HasErrors() {
+		var comparableErr error
+		if errValue, isDiag := diags.Err().(tfdiags.DiagnosticsAsError); isDiag {
+			// To use `As` below we need to extract the error that's wrapped
+			// in a diagnostic.
+			comparableErr = errValue.WrappedErrors()[0]
+		}
+		if !IsA[badChecksumError](comparableErr) {
+			t.Fatalf("expected state checksum error: got %s", diags.Err())
+		} else if bse, ok := As[badChecksumError](comparableErr); ok && len(bse.digest) != 0 {
+			t.Fatalf("expected empty checksum, got %x", bse.digest)
+		}
 	}
 
 	// put the old state in place of the new, without updating the checksum
-	if err := client2.Put(oldState.Bytes()); err != nil {
-		t.Fatal(err)
+	if diags := client2.Put(oldState.Bytes()); diags.HasErrors() {
+		t.Fatal(diags.Err())
 	}
 
 	// fetching the wrong state through client1 should now error out due to a
 	// mismatched checksum.
-	if _, err := client1.Get(); !IsA[badChecksumError](err) {
-		t.Fatalf("expected state checksum error: got %s", err)
+	if _, diags := client1.Get(); diags.HasErrors() {
+		var comparableErr error
+		if errValue, isDiag := diags.Err().(tfdiags.DiagnosticsAsError); isDiag {
+			// To use `As` below we need to extract the error that's wrapped
+			// in a diagnostic.
+			comparableErr = errValue.WrappedErrors()[0]
+		}
+		if !IsA[badChecksumError](comparableErr) {
+			t.Fatalf("expected state checksum error: got %s", diags.Err())
+		}
 	}
 
 	// update the state with the correct one after we Get again
 	testChecksumHook = func() {
-		if err := client2.Put(newState.Bytes()); err != nil {
-			t.Fatal(err)
+		if diags := client2.Put(newState.Bytes()); diags.HasErrors() {
+			t.Fatal(diags.Err())
 		}
 		testChecksumHook = nil
 	}
@@ -415,8 +432,8 @@ func TestRemoteClient_stateChecksum(t *testing.T) {
 	// this final Get will fail to fail the checksum verification, the above
 	// callback will update the state with the correct version, and Get should
 	// retry automatically.
-	if _, err := client1.Get(); err != nil {
-		t.Fatal(err)
+	if _, diags := client1.Get(); diags.HasErrors() {
+		t.Fatal(diags.Err())
 	}
 }
 
@@ -440,22 +457,22 @@ func TestRemoteClientPutLargeUploadWithObjectLock_Compliance(t *testing.T) {
 	)
 	defer deleteS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 
-	s1, err := b.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, diags := b.StateMgr(backend.DefaultStateName)
+	if diags.HasErrors() {
+		t.Fatal(diags)
 	}
 	client := s1.(*remote.State).Client
 
 	var state bytes.Buffer
 	dataW := io.LimitReader(neverEnding('x'), manager.DefaultUploadPartSize*2)
-	_, err = state.ReadFrom(dataW)
+	_, err := state.ReadFrom(dataW)
 	if err != nil {
 		t.Fatalf("writing dummy data: %s", err)
 	}
 
-	err = client.Put(state.Bytes())
-	if err != nil {
-		t.Fatalf("putting data: %s", err)
+	diags = client.Put(state.Bytes())
+	if diags.HasErrors() {
+		t.Fatalf("putting data: %s", diags.Err())
 	}
 }
 
@@ -480,22 +497,22 @@ func TestRemoteClientLockFileWithObjectLock_Compliance(t *testing.T) {
 	)
 	defer deleteS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 
-	s1, err := b.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, diags := b.StateMgr(backend.DefaultStateName)
+	if diags.HasErrors() {
+		t.Fatal(diags)
 	}
 	client := s1.(*remote.State).Client
 
 	var state bytes.Buffer
 	dataW := io.LimitReader(neverEnding('x'), manager.DefaultUploadPartSize)
-	_, err = state.ReadFrom(dataW)
+	_, err := state.ReadFrom(dataW)
 	if err != nil {
 		t.Fatalf("writing dummy data: %s", err)
 	}
 
-	err = client.Put(state.Bytes())
-	if err != nil {
-		t.Fatalf("putting data: %s", err)
+	diags = client.Put(state.Bytes())
+	if diags.HasErrors() {
+		t.Fatalf("putting data: %s", diags.Err())
 	}
 }
 
@@ -520,22 +537,22 @@ func TestRemoteClientLockFileWithObjectLock_Governance(t *testing.T) {
 	)
 	defer deleteS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 
-	s1, err := b.StateMgr(backend.DefaultStateName)
-	if err != nil {
-		t.Fatal(err)
+	s1, diags := b.StateMgr(backend.DefaultStateName)
+	if diags.HasErrors() {
+		t.Fatal(diags)
 	}
 	client := s1.(*remote.State).Client
 
 	var state bytes.Buffer
 	dataW := io.LimitReader(neverEnding('x'), manager.DefaultUploadPartSize)
-	_, err = state.ReadFrom(dataW)
+	_, err := state.ReadFrom(dataW)
 	if err != nil {
 		t.Fatalf("writing dummy data: %s", err)
 	}
 
-	err = client.Put(state.Bytes())
-	if err != nil {
-		t.Fatalf("putting data: %s", err)
+	diags = client.Put(state.Bytes())
+	if diags.HasErrors() {
+		t.Fatalf("putting data: %s", diags.Err())
 	}
 }
 
@@ -590,9 +607,9 @@ func TestRemoteClientSkipS3Checksum(t *testing.T) {
 			createS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 			defer deleteS3Bucket(ctx, t, b.s3Client, bucketName, b.awsConfig.Region)
 
-			state, err := b.StateMgr(backend.DefaultStateName)
-			if err != nil {
-				t.Fatal(err)
+			state, sDiags := b.StateMgr(backend.DefaultStateName)
+			if sDiags.HasErrors() {
+				t.Fatal(sDiags)
 			}
 
 			c := state.(*remote.State).Client
@@ -606,7 +623,7 @@ func TestRemoteClientSkipS3Checksum(t *testing.T) {
 			}
 
 			var checksum string
-			err = client.put(stateBuf.Bytes(), func(opts *s3.Options) {
+			err := client.put(stateBuf.Bytes(), func(opts *s3.Options) {
 				opts.APIOptions = append(opts.APIOptions,
 					addRetrieveChecksumHeaderMiddleware(t, &checksum),
 					addCancelRequestMiddleware(),
