@@ -79,21 +79,21 @@ func TestBackendStates(t *testing.T, b Backend) {
 	t.Helper()
 
 	noDefault := false
-	if _, err := b.StateMgr(DefaultStateName); err != nil {
-		if err == ErrDefaultWorkspaceNotSupported {
+	if _, sDiags := b.StateMgr(DefaultStateName); sDiags.HasErrors() {
+		if sDiags.Err().Error() == ErrDefaultWorkspaceNotSupported.Error() {
 			noDefault = true
 		} else {
-			t.Fatalf("error: %v", err)
+			t.Fatalf("error: %v", sDiags.Err())
 		}
 	}
 
-	workspaces, err := b.Workspaces()
-	if err != nil {
-		if err == ErrWorkspacesNotSupported {
+	workspaces, wDiag := b.Workspaces()
+	if wDiag.HasErrors() {
+		if wDiag.Err().Error() == ErrWorkspacesNotSupported.Error() {
 			t.Logf("TestBackend: workspaces not supported in %T, skipping", b)
 			return
 		}
-		t.Fatalf("error: %v", err)
+		t.Fatalf("error: %v", wDiag.Err())
 	}
 
 	// Test it starts with only the default
@@ -102,9 +102,9 @@ func TestBackendStates(t *testing.T, b Backend) {
 	}
 
 	// Create a couple states
-	foo, err := b.StateMgr("foo")
-	if err != nil {
-		t.Fatalf("error: %s", err)
+	foo, sDiags := b.StateMgr("foo")
+	if sDiags.HasErrors() {
+		t.Fatalf("error: %s", sDiags.Err())
 	}
 	if err := foo.RefreshState(); err != nil {
 		t.Fatalf("bad: %s", err)
@@ -113,9 +113,9 @@ func TestBackendStates(t *testing.T, b Backend) {
 		t.Fatalf("should be empty: %s", v)
 	}
 
-	bar, err := b.StateMgr("bar")
-	if err != nil {
-		t.Fatalf("error: %s", err)
+	bar, sDiags := b.StateMgr("bar")
+	if sDiags.HasErrors() {
+		t.Fatalf("error: %s", sDiags.Err())
 	}
 	if err := bar.RefreshState(); err != nil {
 		t.Fatalf("bad: %s", err)
@@ -177,9 +177,9 @@ func TestBackendStates(t *testing.T, b Backend) {
 		}
 
 		// fetch foo again from the backend
-		foo, err = b.StateMgr("foo")
-		if err != nil {
-			t.Fatal("error re-fetching state:", err)
+		foo, sDiags = b.StateMgr("foo")
+		if sDiags.HasErrors() {
+			t.Fatalf("error re-fetching state: %s", sDiags.Err())
 		}
 		if err := foo.RefreshState(); err != nil {
 			t.Fatal("error refreshing foo:", err)
@@ -190,9 +190,9 @@ func TestBackendStates(t *testing.T, b Backend) {
 		}
 
 		// fetch the bar again from the backend
-		bar, err = b.StateMgr("bar")
-		if err != nil {
-			t.Fatal("error re-fetching state:", err)
+		bar, sDiags = b.StateMgr("bar")
+		if sDiags.HasErrors() {
+			t.Fatalf("error re-fetching state: %s", sDiags.Err())
 		}
 		if err := bar.RefreshState(); err != nil {
 			t.Fatal("error refreshing bar:", err)
@@ -206,9 +206,9 @@ func TestBackendStates(t *testing.T, b Backend) {
 	// Verify we can now list them
 	{
 		// we determined that named stated are supported earlier
-		workspaces, err := b.Workspaces()
-		if err != nil {
-			t.Fatalf("err: %s", err)
+		workspaces, diags := b.Workspaces()
+		if diags.HasErrors() {
+			t.Fatalf("err: %s", diags.Err())
 		}
 
 		sort.Strings(workspaces)
@@ -222,21 +222,21 @@ func TestBackendStates(t *testing.T, b Backend) {
 	}
 
 	// Delete some workspaces
-	if err := b.DeleteWorkspace("foo", true); err != nil {
-		t.Fatalf("err: %s", err)
+	if diags := b.DeleteWorkspace("foo", true); diags.HasErrors() {
+		t.Fatalf("err: %s", diags.Err())
 	}
 
 	// Verify the default state can't be deleted
-	if err := b.DeleteWorkspace(DefaultStateName, true); err == nil {
+	if diags := b.DeleteWorkspace(DefaultStateName, true); !diags.HasErrors() {
 		t.Fatal("expected error")
 	}
 
 	// Create and delete the foo workspace again.
 	// Make sure that there are no leftover artifacts from a deleted state
 	// preventing re-creation.
-	foo, err = b.StateMgr("foo")
-	if err != nil {
-		t.Fatalf("error: %s", err)
+	foo, sDiags = b.StateMgr("foo")
+	if sDiags.HasErrors() {
+		t.Fatalf("error: %s", sDiags)
 	}
 	if err := foo.RefreshState(); err != nil {
 		t.Fatalf("bad: %s", err)
@@ -245,15 +245,15 @@ func TestBackendStates(t *testing.T, b Backend) {
 		t.Fatalf("should be empty: %s", v)
 	}
 	// and delete it again
-	if err := b.DeleteWorkspace("foo", true); err != nil {
-		t.Fatalf("err: %s", err)
+	if diags := b.DeleteWorkspace("foo", true); diags.HasErrors() {
+		t.Fatalf("err: %s", diags.Err())
 	}
 
 	// Verify deletion
 	{
-		workspaces, err := b.Workspaces()
-		if err != nil {
-			t.Fatalf("err: %s", err)
+		workspaces, wDiags := b.Workspaces()
+		if wDiags.HasErrors() {
+			t.Fatalf("err: %s", wDiags.Err())
 		}
 
 		sort.Strings(workspaces)
@@ -307,9 +307,9 @@ func testLocksInWorkspace(t *testing.T, b1, b2 Backend, testForceUnlock bool, wo
 	t.Helper()
 
 	// Get the default state for each
-	b1StateMgr, err := b1.StateMgr(DefaultStateName)
-	if err != nil {
-		t.Fatalf("error: %s", err)
+	b1StateMgr, sDiags := b1.StateMgr(DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatalf("error: %s", sDiags)
 	}
 	if err := b1StateMgr.RefreshState(); err != nil {
 		t.Fatalf("bad: %s", err)
@@ -323,9 +323,9 @@ func testLocksInWorkspace(t *testing.T, b1, b2 Backend, testForceUnlock bool, wo
 
 	t.Logf("TestBackend: testing state locking for %T", b1)
 
-	b2StateMgr, err := b2.StateMgr(DefaultStateName)
-	if err != nil {
-		t.Fatalf("error: %s", err)
+	b2StateMgr, sDiags := b2.StateMgr(DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Fatalf("error: %s", sDiags)
 	}
 	if err := b2StateMgr.RefreshState(); err != nil {
 		t.Fatalf("bad: %s", err)
@@ -351,9 +351,9 @@ func testLocksInWorkspace(t *testing.T, b1, b2 Backend, testForceUnlock bool, wo
 	// Make sure we can still get the statemgr.Full from another instance even
 	// when locked.  This should only happen when a state is loaded via the
 	// backend, and as a remote state.
-	_, err = b2.StateMgr(DefaultStateName)
-	if err != nil {
-		t.Errorf("failed to read locked state from another backend instance: %s", err)
+	_, sDiags = b2.StateMgr(DefaultStateName)
+	if sDiags.HasErrors() {
+		t.Errorf("failed to read locked state from another backend instance: %s", sDiags)
 	}
 
 	// If the lock ID is blank, assume locking is disabled
