@@ -229,8 +229,8 @@ func (c *ApplyCommand) PrepareBackend(planFile *planfile.WrappedPlanFile, args *
 		}
 
 		be, beDiags = c.Backend(&BackendOpts{
-			Config:   backendConfig,
-			ViewType: viewType,
+			BackendConfig: backendConfig,
+			ViewType:      viewType,
 		})
 	}
 
@@ -274,6 +274,7 @@ func (c *ApplyCommand) OperationRequest(
 	opReq.Type = backendrun.OperationTypeApply
 	opReq.View = view.Operation()
 	opReq.StatePersistInterval = c.Meta.StatePersistInterval()
+	opReq.ActionTargets = args.ActionTargets
 
 	// EXPERIMENTAL: maybe enable deferred actions
 	if c.AllowExperimentalFeatures {
@@ -384,10 +385,15 @@ Options:
 
   -state=path            Path to read and save state (unless state-out
                          is specified). Defaults to "terraform.tfstate".
+                         Legacy option for the local backend only. See the local
+                         backend's documentation for more information.
 
   -state-out=path        Path to write state to that is different than
                          "-state". This can be used to preserve the old
                          state.
+                         Legacy option for the local backend only. See the local
+                         backend's documentation for more information.
+
                          
   -var 'foo=bar'         Set a value for one of the input variables in the root
                          module of the configuration. Use this option more than
