@@ -59,6 +59,48 @@ func (c *ResourceInstanceChange) String() string {
 	return fmt.Sprintf("%s: Plan to %s", c.Resource.Addr, c.Action)
 }
 
+func NewPlannedActionInvocation(aiSrc *plans.ActionInvocationInstanceSrc) *ActionInvocation {
+	ai := &ActionInvocation{
+		Action: newActionAddr(aiSrc.Addr),
+	}
+
+	switch trigger := aiSrc.ActionTrigger.(type) {
+	case *plans.LifecycleActionTrigger:
+		ai.LifecycleTrigger = &ActionInvocationLifecycleTrigger{
+			TriggeringResource:      newResourceAddr(trigger.TriggeringResourceAddr),
+			TriggeringEvent:         trigger.ActionTriggerEvent.String(),
+			ActionTriggerBlockIndex: trigger.ActionTriggerBlockIndex,
+			ActionsListIndex:        trigger.ActionsListIndex,
+		}
+	case *plans.InvokeActionTrigger:
+		ai.InvokeTrigger = new(ActionInvocationInvokeTrigger)
+	}
+
+	return ai
+}
+
+type ActionInvocation struct {
+	Action           ActionAddr                        `json:"action_addr"`
+	LifecycleTrigger *ActionInvocationLifecycleTrigger `json:"lifecycle_trigger,omitempty"`
+	InvokeTrigger    *ActionInvocationInvokeTrigger    `json:"action_trigger,omitempty"`
+}
+
+func (c *ActionInvocation) String() string {
+	if c.LifecycleTrigger != nil {
+		return fmt.Sprintf("%s: triggered by %s trigger on %s", c.Action.Addr, c.LifecycleTrigger.TriggeringEvent, c.LifecycleTrigger.TriggeringResource)
+	}
+	return c.Action.Addr
+}
+
+type ActionInvocationLifecycleTrigger struct {
+	TriggeringResource      ResourceAddr `json:"triggering_resource"`
+	TriggeringEvent         string       `json:"triggering_event"`
+	ActionTriggerBlockIndex int          `json:"action_trigger_block_index"`
+	ActionsListIndex        int          `json:"actions_list_index"`
+}
+
+type ActionInvocationInvokeTrigger struct{}
+
 type ChangeAction string
 
 const (
