@@ -14,7 +14,7 @@ import (
 // "Working directory" is unfortunately a slight misnomer, because non-default
 // options can potentially stretch the definition such that multiple working
 // directories end up appearing to share a data directory, or other similar
-// anomolies, but we continue to use this terminology both for historical
+// anomalies, but we continue to use this terminology both for historical
 // reasons and because it reflects the common case without any special
 // overrides.
 //
@@ -135,6 +135,12 @@ func (d *Dir) DataDir() string {
 	return d.dataDir
 }
 
+// TestDataDir returns the path where the receiver keeps settings
+// and artifacts related to terraform tests.
+func (d *Dir) TestDataDir() string {
+	return filepath.Join(d.dataDir, "test")
+}
+
 // ensureDataDir creates the data directory and all of the necessary parent
 // directories that lead to it, if they don't already exist.
 //
