@@ -27,6 +27,10 @@ type ChangesSrc struct {
 
 	Queries []*QueryInstanceSrc
 
+	// ActionInvocations tracks planned action invocations, which may have
+	// embedded resource instance changes.
+	ActionInvocations []*ActionInvocationInstanceSrc
+
 	// Outputs tracks planned changes output values.
 	//
 	// Note that although an in-memory plan contains planned changes for
@@ -59,6 +63,11 @@ func (c *ChangesSrc) Empty() bool {
 		}
 	}
 
+	if len(c.ActionInvocations) > 0 {
+		// action invocations can be applied
+		return false
+	}
+
 	return true
 }
 
@@ -155,6 +164,24 @@ func (c *ChangesSrc) Decode(schemas *schemarepo.Schemas) (*Changes, error) {
 		changes.Queries = append(changes.Queries, query)
 	}
 
+	for _, ais := range c.ActionInvocations {
+		p, ok := schemas.Providers[ais.ProviderAddr.Provider]
+		if !ok {
+			return nil, fmt.Errorf("ChangesSrc.Decode: missing provider %s for %s", ais.ProviderAddr, ais.Addr)
+		}
+		schema, ok := p.Actions[ais.Addr.Action.Action.Type]
+		if !ok {
+			return nil, fmt.Errorf("ChangesSrc.Decode: missing schema for %s", ais.Addr.Action.Action.Type)
+		}
+
+		action, err := ais.Decode(&schema)
+		if err != nil {
+			return nil, err
+		}
+
+		changes.ActionInvocations = append(changes.ActionInvocations, action)
+	}
+
 	for _, ocs := range c.Outputs {
 		oc, err := ocs.Decode()
 		if err != nil {
@@ -180,7 +207,7 @@ type QueryInstanceSrc struct {
 	Addr         addrs.AbsResourceInstance
 	ProviderAddr addrs.AbsProviderConfig
 	Results      DynamicValue
-	Generated    *genconfig.Resource
+	Generated    genconfig.ImportGroup
 }
 
 func (qis *QueryInstanceSrc) Decode(schema providers.Schema) (*QueryInstance, error) {
@@ -530,3 +557,88 @@ func (cs *ChangeSrc) Decode(schema *providers.Schema) (*Change, error) {
 		GeneratedConfig: cs.GeneratedConfig,
 	}, nil
 }
+
+// AppendActionInvocationInstanceChange records the given resource instance change in
+// the set of planned resource changes.
+func (c *ChangesSrc) AppendActionInvocationInstanceChange(action *ActionInvocationInstanceSrc) {
+	if c == nil {
+		panic("AppendActionInvocationInstanceChange on nil ChangesSync")
+	}
+
+	a := action.DeepCopy()
+	c.ActionInvocations = append(c.ActionInvocations, a)
+}
+
+type ActionInvocationInstanceSrc struct {
+	Addr          addrs.AbsActionInstance
+	ActionTrigger ActionTrigger
+
+	ConfigValue          DynamicValue
+	SensitiveConfigPaths []cty.Path
+
+	ProviderAddr addrs.AbsProviderConfig
+}
+
+// Decode unmarshals the raw representation of actions.
+func (acs *ActionInvocationInstanceSrc) Decode(schema *providers.ActionSchema) (*ActionInvocationInstance, error) {
+	ty := cty.DynamicPseudoType
+	if schema != nil {
+		ty = schema.ConfigSchema.ImpliedType()
+	}
+
+	config, err := acs.ConfigValue.Decode(ty)
+	if err != nil {
+		return nil, fmt.Errorf("error decoding 'config' value: %s", err)
+	}
+	markedConfigValue := marks.MarkPaths(config, marks.Sensitive, acs.SensitiveConfigPaths)
+
+	ai := &ActionInvocationInstance{
+		Addr:          acs.Addr,
+		ActionTrigger: acs.ActionTrigger,
+		ProviderAddr:  acs.ProviderAddr,
+		ConfigValue:   markedConfigValue,
+	}
+	return ai, nil
+}
+
+func (acs *ActionInvocationInstanceSrc) DeepCopy() *ActionInvocationInstanceSrc {
+	if acs == nil {
+		return acs
+	}
+	ret := *acs
+	ret.ConfigValue = ret.ConfigValue.Copy()
+	return &ret
+}
+
+func (acs *ActionInvocationInstanceSrc) Less(other *ActionInvocationInstanceSrc) bool {
+	if acs.ActionTrigger.Equals(other.ActionTrigger) {
+		return acs.Addr.Less(other.Addr)
+	}
+	return acs.ActionTrigger.Less(other.ActionTrigger)
+}
+
+// FilterLaterActionInvocations returns the list of action invocations that
+// should be triggered after this one. This function assumes the supplied list
+// of action invocations has already been filtered to invocations against the
+// same resource as the current invocation.
+func (acs *ActionInvocationInstanceSrc) FilterLaterActionInvocations(actionInvocations []*ActionInvocationInstanceSrc) []*ActionInvocationInstanceSrc {
+	needleLat := acs.ActionTrigger.(*LifecycleActionTrigger)
+
+	var laterInvocations []*ActionInvocationInstanceSrc
+	for _, invocation := range actionInvocations {
+		if lat, ok := invocation.ActionTrigger.(*LifecycleActionTrigger); ok {
+			if sameTriggerEvent(lat, needleLat) && triggersLater(lat, needleLat) {
+				laterInvocations = append(laterInvocations, invocation)
+			}
+		}
+	}
+	return laterInvocations
+}
+
+func sameTriggerEvent(one, two *LifecycleActionTrigger) bool {
+	return one.ActionTriggerEvent == two.ActionTriggerEvent
+}
+
+func triggersLater(one, two *LifecycleActionTrigger) bool {
+	return one.ActionTriggerBlockIndex > two.ActionTriggerBlockIndex || (one.ActionTriggerBlockIndex == two.ActionTriggerBlockIndex && one.ActionsListIndex > two.ActionsListIndex)
+}
