@@ -4,6 +4,8 @@
 package providers
 
 import (
+	"iter"
+
 	"github.com/zclconf/go-cty/cty"
 
 	"github.com/hashicorp/terraform/internal/configs/configschema"
@@ -85,6 +87,11 @@ type Interface interface {
 	// ImportResourceState requests that the given resource be imported.
 	ImportResourceState(ImportResourceStateRequest) ImportResourceStateResponse
 
+	// GenerateResourceConfig sends a resource state to the provider, and
+	// expects the provider to return an object which represents a valid
+	// configuration.
+	GenerateResourceConfig(GenerateResourceConfigRequest) GenerateResourceConfigResponse
+
 	// MoveResourceState retrieves the updated value for a resource after it
 	// has moved resource types.
 	MoveResourceState(MoveResourceStateRequest) MoveResourceStateResponse
@@ -121,6 +128,15 @@ type Interface interface {
 	// DeleteState instructs a given state store to delete a specific state (i.e. a CE workspace)
 	DeleteState(DeleteStateRequest) DeleteStateResponse
 
+	// PlanAction plans an action to be invoked, providers might indicate potential drift and
+	// raise issues with the action configuration.
+	PlanAction(PlanActionRequest) PlanActionResponse
+	// InvokeAction invokes an action, providers return a stream of events that update terraform
+	// about the status of the action.
+	InvokeAction(InvokeActionRequest) InvokeActionResponse
+	// ValidateActionConfig performs configuration validation
+	ValidateActionConfig(ValidateActionConfigRequest) ValidateActionConfigResponse
+
 	// Close shuts down the plugin process if applicable.
 	Close() error
 }
@@ -158,6 +174,9 @@ type GetProviderSchemaResponse struct {
 	// StateStores maps the state store type name to that type's schema.
 	StateStores map[string]Schema
 
+	// Actions maps the name of the action to its schema.
+	Actions map[string]ActionSchema
+
 	// Diagnostics contains any warnings or errors from the method call.
 	Diagnostics tfdiags.Diagnostics
 
@@ -184,6 +203,36 @@ type IdentitySchema struct {
 	Body *configschema.Object
 }
 
+type ExecutionOrder string
+
+const (
+	ExecutionOrderInvalid ExecutionOrder = "invalid"
+	ExecutionOrderBefore  ExecutionOrder = "before"
+	ExecutionOrderAfter   ExecutionOrder = "after"
+)
+
+type ActionSchema struct {
+	ConfigSchema *configschema.Block
+}
+
+// IsNil() returns true if there is no action schema at all.
+func (a ActionSchema) IsNil() bool {
+	return a.ConfigSchema == nil
+}
+
+type ListResourceSchema struct {
+	// schema for the nested "config" block.
+	ConfigSchema *configschema.Block
+
+	// schema for the entire block (including "config" block)
+	FullSchema *configschema.Block
+}
+
+// IsNil() returns true if there is no list resource schema at all.
+func (l ListResourceSchema) IsNil() bool {
+	return l.FullSchema == nil
+}
+
 // Schema pairs a provider or resource schema with that schema's version.
 // This is used to be able to upgrade the schema in UpgradeResourceState.
 //
@@ -216,6 +265,11 @@ type ServerCapabilities struct {
 	// The MoveResourceState capability indicates that this provider supports
 	// the MoveResourceState RPC.
 	MoveResourceState bool
+
+	// GenerateResourceConfig indicates that the provider can take an existing
+	// state for a resource instance, and return the subset of the state which
+	// can be used as configuration.
+	GenerateResourceConfig bool
 }
 
 // ClientCapabilities allows Terraform to publish information regarding
@@ -448,7 +502,7 @@ type ReadResourceResponse struct {
 	Private []byte
 
 	// Deferred if present signals that the provider was not able to fully
-	// complete this operation and a susequent run is required.
+	// complete this operation and a subsequent run is required.
 	Deferred *Deferred
 
 	// Identity is the object-typed value representing the identity of the remote
@@ -517,7 +571,7 @@ type PlanResourceChangeResponse struct {
 	LegacyTypeSystem bool
 
 	// Deferred if present signals that the provider was not able to fully
-	// complete this operation and a susequent run is required.
+	// complete this operation and a subsequent run is required.
 	Deferred *Deferred
 
 	// PlannedIdentity is the planned identity data of the resource.
@@ -604,10 +658,24 @@ type ImportResourceStateResponse struct {
 	Diagnostics tfdiags.Diagnostics
 
 	// Deferred if present signals that the provider was not able to fully
-	// complete this operation and a susequent run is required.
+	// complete this operation and a subsequent run is required.
 	Deferred *Deferred
 }
 
+// GenerateResourceConfigRequest contains the most recent state of a resource
+// instance which the provider can use to generate a valid configuration object.
+type GenerateResourceConfigRequest struct {
+	TypeName string
+	State    cty.Value
+}
+
+type GenerateResourceConfigResponse struct {
+	// Config is the subset of the resource state which represents a valid
+	// configuration object for the instance.
+	Config      cty.Value
+	Diagnostics tfdiags.Diagnostics
+}
+
 // ImportedResource represents an object being imported into Terraform with the
 // help of a provider. An ImportedResource is a RemoteObject that has been read
 // by the provider's import handler but hasn't yet been committed to state.
@@ -701,7 +769,7 @@ type ReadDataSourceResponse struct {
 	Diagnostics tfdiags.Diagnostics
 
 	// Deferred if present signals that the provider was not able to fully
-	// complete this operation and a susequent run is required.
+	// complete this operation and a subsequent run is required.
 	Deferred *Deferred
 }
 
@@ -730,7 +798,7 @@ type CallFunctionResponse struct {
 	// provider schema.
 	//
 	// If Diagnostics contains any errors, this field will be ignored and
-	// so can be left as cty.NilVal to represent the absense of a value.
+	// so can be left as cty.NilVal to represent the absence of a value.
 	Result cty.Value
 
 	// Err is the error value from the function call. This may be an instance
@@ -812,3 +880,62 @@ type DeleteStateResponse struct {
 	// Diagnostics contains any warnings or errors from the method call.
 	Diagnostics tfdiags.Diagnostics
 }
+
+type PlanActionRequest struct {
+	ActionType         string
+	ProposedActionData cty.Value
+
+	ClientCapabilities ClientCapabilities
+}
+
+type PlanActionResponse struct {
+	Deferred    *Deferred
+	Diagnostics tfdiags.Diagnostics
+}
+
+type InvokeActionRequest struct {
+	ActionType         string
+	PlannedActionData  cty.Value
+	ClientCapabilities ClientCapabilities
+}
+
+type InvokeActionResponse struct {
+	Events      iter.Seq[InvokeActionEvent]
+	Diagnostics tfdiags.Diagnostics
+}
+
+type InvokeActionEvent interface {
+	isInvokeActionEvent()
+}
+
+// Completed Event
+var _ InvokeActionEvent = &InvokeActionEvent_Completed{}
+
+type InvokeActionEvent_Completed struct {
+	Diagnostics tfdiags.Diagnostics
+}
+
+func (e InvokeActionEvent_Completed) isInvokeActionEvent() {}
+
+// Progress Event
+var _ InvokeActionEvent = &InvokeActionEvent_Progress{}
+
+type InvokeActionEvent_Progress struct {
+	Message string
+}
+
+func (e InvokeActionEvent_Progress) isInvokeActionEvent() {}
+
+type ValidateActionConfigRequest struct {
+	// TypeName is the name of the action type to validate.
+	TypeName string
+
+	// Config is the configuration value to validate, which may contain unknown
+	// values.
+	Config cty.Value
+}
+
+type ValidateActionConfigResponse struct {
+	// Diagnostics contains any warnings or errors from the method call.
+	Diagnostics tfdiags.Diagnostics
+}
