@@ -284,3 +284,86 @@ func TestOmittingBuiltInProviders(t *testing.T) {
 		})
 	})
 }
+
+func TestComponentSourceResolution(t *testing.T) {
+	stackResourceName := "pet-nulls"
+	bundle, err := sourcebundle.OpenDir("testdata/embedded-stack-bundle")
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	rootAddr := sourceaddrs.MustParseSource("git::https://example.com/root.git").(sourceaddrs.RemoteSource)
+	config, diags := LoadConfigDir(rootAddr, bundle)
+	if len(diags) != 0 {
+		t.Fatalf("unexpected diagnostics:\n%s", diags.NonFatalErr().Error())
+	}
+
+	t.Run("component source resolution", func(t *testing.T) {
+		// Verify that the component was loaded
+		if got, want := len(config.Root.Stack.EmbeddedStacks), 1; got != want {
+			t.Errorf("wrong number of components %d; want %d", got, want)
+		}
+
+		t.Run("pet-nulls component", func(t *testing.T) {
+			cmpn, ok := config.Root.Stack.EmbeddedStacks[stackResourceName]
+			if !ok {
+				t.Fatalf("Root stack config has no component named %q.", stackResourceName)
+			}
+
+			// Verify component name
+			if got, want := cmpn.Name, stackResourceName; got != want {
+				t.Errorf("wrong component name\ngot:  %s\nwant: %s", got, want)
+			}
+
+			// Verify that the source address was parsed correctly
+			componentSource, ok := cmpn.SourceAddr.(sourceaddrs.ComponentSource)
+			if !ok {
+				t.Fatalf("expected ComponentSource, got %T", cmpn.SourceAddr)
+			}
+
+			expectedSourceStr := "example.com/awesomecorp/tfstack-pet-nulls"
+			if got := componentSource.String(); got != expectedSourceStr {
+				t.Errorf("wrong source address\ngot:  %s\nwant: %s", got, expectedSourceStr)
+			}
+
+			// Verify that version constraints were parsed
+			if cmpn.VersionConstraints == nil {
+				t.Fatal("component has no version constraints")
+			}
+
+			// Verify that the final source address was resolved
+			if cmpn.FinalSourceAddr == nil {
+				t.Fatal("component FinalSourceAddr was not resolved")
+			}
+
+			// The final source should be a ComponentSourceFinal
+			componentSourceFinal, ok := cmpn.FinalSourceAddr.(sourceaddrs.ComponentSourceFinal)
+			if !ok {
+				t.Fatalf("expected ComponentSourceFinal for FinalSourceAddr, got %T", cmpn.FinalSourceAddr)
+			}
+
+			// Verify it resolved to the correct version (0.0.2)
+			expectedVersion := "0.0.2"
+			if got := componentSourceFinal.SelectedVersion().String(); got != expectedVersion {
+				t.Errorf("wrong selected version\ngot:  %s\nwant: %s", got, expectedVersion)
+			}
+
+			// Verify the unversioned component source matches
+			if got := componentSourceFinal.Unversioned().String(); got != expectedSourceStr {
+				t.Errorf("wrong unversioned source in final address\ngot:  %s\nwant: %s", got, expectedSourceStr)
+			}
+
+			// Verify we can get the local path from the bundle
+			localPath, err := bundle.LocalPathForSource(cmpn.FinalSourceAddr)
+			if err != nil {
+				t.Fatalf("failed to get local path for component source: %s", err)
+			}
+
+			// The local path should point to the pet-nulls directory
+			if localPath == "" {
+				t.Error("local path is empty")
+			}
+			t.Logf("Component resolved to local path: %s", localPath)
+		})
+	})
+}
